require 'sinatra/base'

module BigcommerceTool
  class App < Sinatra::Base

    set :template_folder, './template'
    set :override_templates_folder, './config/templates'
    set :global_variables, './config/global.yml'
    set :global_files, './config/global_files'
    set :language_variables, './config/language.yml'
    set :page_variables, './config/page.yml'
    set :discount_variables, './config/discount.yml'

    configure do
      mime_type :javascript, 'text/javascript'
      mime_type :css, 'text/css'
      mime_type :png, 'image/png'
      mime_type :svg, 'image/svg+xml'
      mime_type :jpg, 'image/jpeg'
      mime_type :woff, 'application/x-font-woff'
      mime_type :gif, 'image/gif'
    end

    def mime_type_by_file(file)
      type = file.split('.').last.downcase
      case type
      when 'js'
        :javascript
      when 'css'
        :css
      when 'svg'
        :svg
      when 'png'
        :png
      when 'jpeg','jpg'
        :jpg
      when 'woff','woff2'
        :woff
      when 'gif'
        :gif
      end
    end

    def template
      @template ||= BigcommerceTool::Template.new(settings.template_folder,
                                                    settings.override_templates_folder,
                                                    settings.global_variables,
                                                    settings.global_files,
                                                    settings.language_variables,
                                                    settings.page_variables,
                                                    settings.discount_variables)
    end

    def template_results(page)
      rtn = template.render(page)
      unless rtn.empty?
        rtn
      else
        missing_page(page)
      end
    end

    def missing_page(page)
      halt(404, "404, No page found for #{page}")
    end

    def existing_variables(file_path)
      puts template.extract_variables(File.open(file_path, "rb").read).inspect
    end

    def remote(params)
      puts "params: #{params.inspect}"
      json_path = File.join('config', 'json_responses', "#{params[:w]}.json")
      if File.exists?(json_path)
        template.load_file(json_path)
      else
        missing_page('remote.php: '+params.inspect)
      end
    end

    helpers do
      def json_template(template_name)
        template.process_file(template.get_snippet(template_name).inspect).gsub(/^"|"$/, '')
      end
    end

    post '/remote.php' do
      #@template = self.template
      erb remote(params)
    end

    get '/cms/:name' do
      template.cms(params[:name])
    end

    get '/' do
      template_results('default.html')
    end

    get '/:name' do
      template_results(params[:name].gsub(/\.php$/, '.html'))
    end

    get '/Emails/:name' do
      template_results("Emails/#{params[:name]}")
    end

    get %r{/(js|Styles|fonts|images)/(.+)} do
      content_type mime_type_by_file(params[:captures].last)
      template_results("#{params[:captures].first}/#{params[:captures].last}")
    end
  end
end
