module BigcommerceTool
  class Deployer
    def initialize(options)
      raise 'Missing config file' unless File.exists?('bigcommerce.yml') 
      @branch = options[:branch] || ENV['CIRCLE_BRANCH']
      @config = YAML.load_file(options[:config_path] || 'bigcommerce.yml')
    end

    def deploy
      return unless env_name

      Net::DAV.start(URI("https://#{url}/dav/")) do |dav|
        dav.credentials(ENV['USERNAME'], password)
        puts "push to: #{url}"
        changed_files.each do |file|
          begin
            if File.exists?(file)
              if file.include?(' ')
                puts "skipping: #{file}, file has a space in the name."
                next
              else
                create_path_if_missing(dav, file)
                dav.put_string(file, File.open(file, 'r').read)
                puts "add/update #{file}"
              end
            else
              dav.delete(file) if dav.exists?(file)
              puts "remove: #{file}"
            end
          rescue Exception => e
            puts "file: #{file}"
            puts e.message
            puts e.backtrace
            raise e
          end
        end
      end
    end

  private

    def environments_mapping
      @config['deploy']['environments_mapping']
    end

    def env_name
      environments_mapping[@branch].or_if_nil? do
        puts "Skipping deploy, no mapping set up for branch: #{@branch}"
      end
    end

    def url
      ENV["#{env_name}_URL"]
    end

    def password
      ENV["#{env_name}_PASSWORD"]
    end

    def create_directory(conn, path, arr)
      @current_folders ||= {}
      create_path = (path.empty?) ? arr.shift : "#{path}/#{arr.shift}"
      unless @current_folders.has_key?(create_path)
        conn.mkdir(create_path) unless conn.exists?(create_path)
        @current_folders[create_path] = 1
      end
      create_directory(conn, create_path, arr) if arr.length > 0
    end

    def create_path_if_missing(conn, file)
      create_directory(conn, '', File.dirname(file).split(/\//))
    end

    def changed_files
      all_files = Dir.glob('template/**/*.*')
      return all_files if !ENV['CIRCLE_COMPARE_URL'] || ENV['CIRCLE_COMPARE_URL'].empty?
      puts ENV['CIRCLE_COMPARE_URL']

      range = ENV['CIRCLE_COMPARE_URL'].split('/').last

      `git diff --name-only #{range}`.split("\n").reject{|file| file !~ /^template\//}
    end

  end
end
